@@ -28,7 +28,7 @@ require Storable;
 package RPC::PlServer::Comm;
 
 
-$RPC::PlServer::Comm::VERSION = '0.1001';
+$RPC::PlServer::Comm::VERSION = '0.1000';
 
 
 ############################################################################
@@ -54,7 +54,10 @@ sub Write ($$) {
     my($self, $msg) = @_;
     my $socket = $self->{'socket'};
 
-    my($encodedMsg) = Storable::nfreeze($msg);
+    my $encodedMsg = Storable::nfreeze($msg);
+    $encodedMsg = Compress::Zlib::compress($encodedMsg)
+	if ($self->{'compression'});
+
     my($encodedSize) = length($encodedMsg);
     if (my $cipher = $self->{'cipher'}) {
 	my $size = $cipher->blocksize;
@@ -87,7 +90,6 @@ sub Write ($$) {
 #   Inputs:  $self - Instance of RPC::PlServer or RPC::PlClient
 #
 #   Result:  Array ref to result list; dies in case of errors.
-#	     Undef is returned in case of EOF.
 #
 ############################################################################
 
@@ -102,13 +104,15 @@ sub Read($) {
     while ($readSize > 0) {
 	my $result = $socket->read($encodedSize, $readSize,
 				   length($encodedSize));
-	if (!$result) {
-	    die "Error while reading socket: $!" if !defined $result;
-	    return undef;
+	if (!defined($result)) {
+	    die "Error while reading socket: $!";
 	}
 	$readSize -= $result;
     }
     $encodedSize = unpack("N", $encodedSize);
+    my $max = $self->{'maxmessage'} || (1 << 16);
+    die "Maximum message size of $max exceeded, use option 'maxmessage' to"
+	. " increase" if $encodedSize > $max;
     $readSize = $encodedSize;
     if ($self->{'cipher'}) {
 	$blockSize = $self->{'cipher'}->blocksize;
@@ -120,9 +124,8 @@ sub Read($) {
     my $rs = $readSize;
     while ($rs > 0) {
 	my $result = $socket->read($msg, $rs, length($msg));
-	if (!$result) {
-	    die "Error while reading socket: $!" if !defined $result;
-	    die "Unexpected EOF while waiting for data";
+	if (!defined($result)) {
+	    die "Error while reading socket: $!";
 	}
 	$rs -= $result;
     }
@@ -135,8 +138,36 @@ sub Read($) {
 	}
 	$msg = substr($msg, 0, $encodedSize);
     }
+    $msg = Compress::Zlib::uncompress($msg) if ($self->{'compression'});
     Storable::thaw($msg);
 }
 
 
+############################################################################
+#
+#   Name:    Init
+#
+#   Purpose: Initialize an object for using RPC::PlServer::Comm methods
+#
+#   Input:   $self - Instance
+#
+#   Returns: The instance in case of success, dies in case of trouble.
+#
+############################################################################
+
+sub Init {
+    my $self = shift;
+    if (my $comp = $self->{'compression'}) {
+	if ($comp eq 'off') {
+	    $self->{'compression'} = undef;
+	} elsif ($comp eq 'gzip') {
+	    require Compress::Zlib;
+	} else {
+	    die "Unknown compression type ($comp), use 'off' or 'gzip'";
+	}
+    }
+    $self;
+}
+
+
 1;
